@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: LMfit.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: LMfit.h 322 2007-07-24 00:17:11Z wojdyr $
 
 #ifndef FITYK__LMFIT__H__
 #define FITYK__LMFIT__H__
@@ -14,7 +14,7 @@
 class LMfit : public Fit
 {
 public:
-    LMfit();
+    LMfit(Ftk* F);
     ~LMfit();
     fp init(); // called before do_iteration()/autoiter()
     void autoiter();
