@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) 2005 Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: ast.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: ast.cpp 294 2007-05-16 03:18:25Z wojdyr $
 
 //  Based on ast_calc example from Boost::Spirit by Daniel Nuffer
 
@@ -838,7 +838,7 @@ fp get_constant_value(string const &s)
     else {
         fp val = strtod(s.c_str(), 0);
         if (val != 0. && fabs(val) < epsilon)
-            warn("Warning: Numeric literal 0 < |" + s + "| < epsilon=" 
+            AL->warn("Warning: Numeric literal 0 < |" + s + "| < epsilon=" 
                     + S(epsilon) + ".");
         return val;
     }
