@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: bfunc.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: bfunc.h 322 2007-07-24 00:17:11Z wojdyr $
 #ifndef FITYK__BFUNC__H__
 #define FITYK__BFUNC__H__
 
@@ -11,18 +11,22 @@
 #define DECLARE_FUNC_OBLIGATORY_METHODS(NAME) \
     friend class Function;\
 protected:\
-    Func##NAME (std::string const &name, std::vector<std::string> const &vars,\
+    Func##NAME (Ftk const* F, \
+                std::string const &name, \
+                std::vector<std::string> const &vars, \
                 std::string const &formula_) \
-        : Function(name, vars, formula_) {}\
+        : Function(F, name, vars, formula_) {} \
 private:\
-    Func##NAME (std::string const &name, std::vector<std::string> const &vars)\
-        : Function(name, vars, formula) {}\
+    Func##NAME (Ftk const* F, \
+                std::string const &name, \
+                std::vector<std::string> const &vars) \
+        : Function(F, name, vars, formula) {} \
     Func##NAME (const Func##NAME&); \
 public:\
     static const char *formula; \
     void calculate_value(std::vector<fp> const &xx, std::vector<fp> &yy) const;\
     void calculate_value_deriv(std::vector<fp> const &xx, \
-                               std::vector<fp> &yy, std::vector<fp> &dy_da,\
+                               std::vector<fp> &yy, std::vector<fp> &dy_da, \
                                bool in_dx=false) const; 
 
 //////////////////////////////////////////////////////////////////////////
@@ -161,6 +165,8 @@ class FuncVoigt : public Function
     fp fwhm() const;
     bool has_area() const { return true; } 
     fp area() const;
+    std::vector<std::string> get_other_prop_names() const;
+    fp other_prop(std::string const& name) const;
 };
 
 class FuncVoigtA : public Function
