@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: cmd2.cpp 265 2007-03-02 18:41:10Z wojdyr $
+// $Id: cmd2.cpp 297 2007-05-25 02:33:19Z wojdyr $
 
 /// big grammars in Spirit take a lot of time and memory to compile
 /// so they must be splitted into separate compilation units
@@ -28,6 +28,13 @@
 #include "func.h"
 #include "ast.h"
 
+#ifndef CONFIGURE_BUILD
+# define CONFIGURE_BUILD "UNKNOWN"
+#endif
+#ifndef CONFIGURE_ARGS
+# define CONFIGURE_ARGS "UNKNOWN"
+#endif
+
 using namespace std;
 
 namespace cmdgram {
@@ -43,6 +50,7 @@ vector<int> vn, vds;
 const int new_dataset = -1;
 const int all_datasets = -2;
 bool outdated_plot = false;
+bool no_info_output = false;
 
 
 vector<DataWithSum*> get_datasets_from_indata()
@@ -69,34 +77,11 @@ IntRangeGrammar  IntRangeG;
 
 using namespace cmdgram;
 
-
 namespace {
 
 void do_import_dataset(char const*, char const*)
 {
-    if (tmp_int == new_dataset) {
-        if (AL->get_ds_count() != 1 || AL->get_data(0)->has_any_info()
-                                    || AL->get_sum(0)->has_any_info()) {
-            // load data into new slot
-            auto_ptr<Data> data(new Data);
-            data->load_file(t, t2, vn); 
-            tmp_int = AL->append_ds(data.release());
-        }
-        else { // there is only one and empty slot -- load data there
-            AL->get_data(-1)->load_file(t, t2, vn); 
-            AL->view.set_datasets(vector1(AL->get_ds(0)));
-            AL->view.fit();
-            tmp_int = 0;
-        }
-    }
-    else { // slot number was specified -- load data there
-        AL->get_data(tmp_int)->load_file(t, t2, vn); 
-        if (AL->get_ds_count() == 1) {
-            AL->view.set_datasets(vector1(AL->get_ds(0)));
-            AL->view.fit();
-        }
-    }
-    AL->activate_ds(tmp_int);
+    AL->import_dataset(tmp_int, t, t2, vn);
     outdated_plot=true;  
 }
 
@@ -145,15 +130,17 @@ void do_plot(char const*, char const*)
         AL->view.set_datasets(dsds);
     }
     AL->view.parse_and_set(vr);
-    getUI()->drawPlot(1, true);
+    AL->get_ui()->draw_plot(1, true);
     outdated_plot = false;
 }
 
 void do_output_info(char const*, char const*)
 {
     prepared_info = strip_string(prepared_info);
+    if (no_info_output)
+        return;
     if (output_redir.empty())
-        rmsg(prepared_info);
+        AL->rmsg(prepared_info);
     else {
         ofstream os(output_redir.c_str(), 
                     ios::out | (info_append ? ios::app : ios::trunc));
@@ -172,7 +159,24 @@ void do_print_info(char const* a, char const* b)
     if (s.empty())
         m = "info about what?";
     else if (s == "version") {
-        m = VERSION;
+        if (with_plus) {
+            m = "Version: " VERSION 
+                "\nBuild system type: " CONFIGURE_BUILD
+                "\nConfigured with: " CONFIGURE_ARGS
+#ifdef __GNUC__
+                "\nCompiler: GCC"
+#endif
+#ifdef __VERSION__
+                "\nCompiler version: " __VERSION__ 
+#endif
+                "\nCompilation date: " __DATE__ 
+                "\nBoost.Spirit version: " + S(SPIRIT_VERSION / 0x1000) 
+                                + "." + S(SPIRIT_VERSION % 0x1000 / 0x0100)
+                                + "." + S(SPIRIT_VERSION % 0x0100);
+ 
+        }
+        else
+            m = VERSION;
     }
     else if (s == "variables") {
         if (variables.empty())
@@ -237,24 +241,24 @@ void do_print_info(char const* a, char const* b)
         else if (s.find(".filename") != string::npos)
             m = data->get_filename();
         else
-            m = data->getInfo();
+            m = data->get_info();
     }
     else if (s == "view") {
         m = AL->view.str();
     }
     else if (s == "set")
-        m = getSettings()->print_usage();
+        m = AL->get_settings()->print_usage();
     else if (startswith(s, "fit-history"))
-        m = FitMethodsContainer::getInstance()->param_history_info();
+        m = AL->get_fit_container()->param_history_info();
     else if (startswith(s, "fit"))
-        m = getFit()->getInfo(get_datasets_from_indata());
+        m = AL->get_fit()->get_info(get_datasets_from_indata());
     else if (startswith(s, "errors"))
-        m = getFit()->getErrorInfo(get_datasets_from_indata(), with_plus);
+        m = AL->get_fit()->getErrorInfo(get_datasets_from_indata(), with_plus);
     else if (startswith(s, "peaks")) {
         vector<fp> errors;
         vector<DataWithSum*> v = get_datasets_from_indata();
         if (with_plus) 
-            errors = getFit()->get_symmetric_errors(v);
+            errors = AL->get_fit()->get_symmetric_errors(v);
         for (vector<DataWithSum*>::const_iterator i=v.begin(); i!=v.end(); ++i){
             m += "# " + (*i)->get_data()->get_title() + "\n";
             m += (*i)->get_sum()->get_peak_parameters(errors) + "\n";
@@ -264,12 +268,12 @@ void do_print_info(char const* a, char const* b)
         vector<DataWithSum*> v = get_datasets_from_indata();
         for (vector<DataWithSum*>::const_iterator i=v.begin(); i!=v.end(); ++i){
             m += "# " + (*i)->get_data()->get_title() + "\n";
-            bool gnuplot = getSettings()->get_e("formula-export-style") == 1;
+            bool gnuplot = AL->get_settings()->get_e("formula-export-style")==1;
             m += (*i)->get_sum()->get_formula(!with_plus, gnuplot) + "\n";
         }
     }
     else if (s == "commands")
-        m = getUI()->getCommands().get_info(with_plus);
+        m = AL->get_ui()->get_commands().get_info(with_plus);
     else if (startswith(s, "guess")) {
         vector<DataWithSum*> v = get_datasets_from_indata();
         for (vector<DataWithSum*>::const_iterator i = v.begin(); 
@@ -354,7 +358,7 @@ void do_print_debug_info(char const*, char const*)
         Function const* f = AL->find_function(t);
         m = f->get_bytecode();
     }
-    rmsg(m);
+    AL->rmsg(m);
 }
 
 
@@ -410,7 +414,7 @@ void set_data_title(char const*, char const*)  {
 void do_list_commands(char const*, char const*)
 {
     vector<string> cc 
-        = getUI()->getCommands().get_commands(tmp_int, tmp_int2, with_plus);
+      = AL->get_ui()->get_commands().get_commands(tmp_int, tmp_int2, with_plus);
     prepared_info += "\n" + join_vector(cc, "\n");
 }
 
