@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: cmd2.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: cmd2.h 273 2007-03-18 12:12:10Z wojdyr $
 
 #ifndef FITYK__CMD2__H__
 #define FITYK__CMD2__H__
@@ -31,6 +31,8 @@ extern std::vector<int> vn, vds;
 extern const int new_dataset;
 extern const int all_datasets;
 extern bool outdated_plot;
+extern bool no_info_output;
+extern std::string prepared_info;
 
 std::vector<DataWithSum*> get_datasets_from_indata();
 
