@@ -1,17 +1,19 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: data.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: data.cpp 345 2007-08-21 01:09:15Z wojdyr $
 
 #include "common.h"
 #include "data.h" 
 #include "fileroutines.h"
-#include "ui.h"
+//#include "ui.h"
 #include "numfuncs.h"
 #include "datatrans.h" 
 #include "settings.h" 
+#include "logic.h" 
 
 #include <math.h>
 #include <string.h>
+#include <stdlib.h>
 #include <fstream>
 #include <sstream>
 #include <algorithm>
@@ -39,7 +41,7 @@ string get_file_basename(const string &path)
 }
 
 
-string Data::getInfo() const
+string Data::get_info() const
 {
     string s;
     if (p.empty())
@@ -48,6 +50,8 @@ string Data::getInfo() const
         s = S(p.size()) + " points, " + S(active_p.size()) + " active.";
     if (!filename.empty())
         s += "\nFilename: " + filename;
+    if (!given_cols.empty())
+        s += "\nColumns: " + join_vector(given_cols, ", ");
     if (!title.empty())
         s += "\nData title: " + title;
     if (active_p.size() != p.size())
@@ -102,7 +106,7 @@ void Data::post_load()
         return;
     string inf = S(p.size()) + " points.";
     if (!has_sigma) {
-        int dds = getSettings()->get_e("data-default-sigma");
+        int dds = F->get_settings()->get_e("data-default-sigma");
         if (dds == 's') {
             for (vector<Point>::iterator i = p.begin(); i < p.end(); i++) 
                 i->sigma = i->y > 1. ? sqrt (i->y) : 1.;
@@ -116,9 +120,12 @@ void Data::post_load()
         else
             assert(0);
     }
-    msg(inf);
-    if (title.empty())
+    F->msg(inf);
+    if (title.empty()) {
         title = get_file_basename(filename);
+        if (given_cols.size() >= 2)
+            title += ":" + S(given_cols[0]) + "," + S(given_cols[1]);
+    }
     update_active_p();
     recompute_y_bounds();
 }
@@ -163,39 +170,80 @@ int Data::load_arrays(const vector<fp> &x, const vector<fp> &y,
 
 namespace {
 
+void merge_same_x(vector<Point> &pp, bool avg)
+{
+    int count_same = 1;
+    fp x0;
+    for (int i = pp.size() - 2; i >= 0; --i) {
+        if (count_same == 1)
+            x0 = pp[i+1].x;
+        if (is_eq(pp[i].x, x0)) {
+            pp[i].x += pp[i+1].x;
+            pp[i].y += pp[i+1].y;
+            pp[i].sigma += pp[i+1].sigma;
+            pp[i].is_active = pp[i].is_active || pp[i+1].is_active;
+            pp.erase(pp.begin() + i+1);
+            count_same++;
+            if (i > 0)
+                continue;
+            else
+                i = -1; // to change pp[0]
+        }
+        if (count_same > 1) {
+            pp[i+1].x /= count_same;
+            if (avg) {
+                pp[i+1].y /= count_same;
+                pp[i+1].sigma /= count_same;
+            }
+            count_same = 1;
+        }
+    }
+}
+
+void shirley_bg(vector<Point> &pp, bool remove)
+{
+    const int max_iter = 50;
+    const double max_rdiff = 1e-6;
+    const int n = pp.size();
+    double ya = pp[0].y; // lowest bg
+    double yb = pp[n-1].y; // highest bg
+    double dy = yb - ya;
+    vector<double> B(n, ya);
+    vector<double> PA(n, 0.);
+    double old_A = 0;
+    for (int iter = 0; iter < max_iter; ++iter) {
+        vector<double> Y(n);
+        for (int i = 0; i < n; ++i)
+            Y[i] = pp[i].y - B[i]; 
+        for (int i = 1; i < n; ++i)
+            PA[i] = PA[i-1] + (Y[i] + Y[i-1]) / 2 * (pp[i].x - pp[i-1].x);
+        double rel_diff = old_A != 0. ? abs(PA[n-1] - old_A) / old_A : 1.;
+        if (rel_diff < max_rdiff) 
+            break;
+        old_A = PA[n-1];
+        for (int i = 0; i < n; ++i)
+            B[i] = ya + dy / PA[n-1] * PA[i];
+    }
+    if (remove)
+        for (int i = 0; i < n; ++i) 
+            pp[i].y -= B[i];
+    else
+        for (int i = 0; i < n; ++i)
+            pp[i].y = B[i];
+}
+
 void apply_operation(vector<Point> &pp, string const& op)
 {
     assert (!pp.empty());
     assert (!op.empty());
-    if (op == "sum_same_x" || op == "avg_same_x") {
-        bool avg = (op == "avg_same_x");
-        int count_same = 1;
-        fp x0;
-        for (int i = pp.size() - 2; i >= 0; --i) {
-            if (count_same == 1)
-                x0 = pp[i+1].x;
-            if (is_eq(pp[i].x, x0)) {
-                pp[i].x += pp[i+1].x;
-                pp[i].y += pp[i+1].y;
-                pp[i].sigma += pp[i+1].sigma;
-                pp[i].is_active = pp[i].is_active || pp[i+1].is_active;
-                pp.erase(pp.begin() + i+1);
-                count_same++;
-                if (i > 0)
-                    continue;
-                else
-                    i = -1; // to change pp[0]
-            }
-            if (count_same > 1) {
-                pp[i+1].x /= count_same;
-                if (avg) {
-                    pp[i+1].y /= count_same;
-                    pp[i+1].sigma /= count_same;
-                }
-                count_same = 1;
-            }
-        }
-    }
+    if (op == "sum_same_x")
+        merge_same_x(pp, false);
+    else if (op == "avg_same_x") 
+        merge_same_x(pp, true);
+    else if (op == "shirley_bg")
+        shirley_bg(pp, false);
+    else if (op == "rm_shirley_bg")
+        shirley_bg(pp, true);
     else if (op == "fft") {
         throw ExecuteError("Fourier Transform not implemented yet");
     }
@@ -236,24 +284,132 @@ void Data::load_data_sum(vector<Data const*> const& dd, string const& op)
     post_load();
 }
 
-void Data::load_file (string const& file, string const& type, 
-                     vector<int> const& cols, bool preview)
+void Data::add_one_point(double x, double y, double sigma)
+{
+    Point pt(x, y, sigma);
+    vector<Point>::iterator a = upper_bound(p.begin(), p.end(), pt);
+    int idx = a - p.begin();
+    p.insert(a, pt);
+    active_p.insert(upper_bound(active_p.begin(), active_p.end(), idx), idx);
+    if (pt.y < y_min)
+        y_min = pt.y;
+    if (pt.y > y_max)
+        y_max = pt.y;
+    // (fast) x_step update
+    if (p.size() < 2)
+        x_step = 0.;
+    else if (p.size() == 2)
+        x_step = p[1].x - p[0].x;
+    else if (x_step != 0) {
+        //TODO use tiny_relat_diff
+        fp max_diff = 1e-4 * fabs(x_step);
+        if (idx == 0 && fabs((p[1].x - p[0].x) - x_step) < max_diff)
+            ; //nothing, the same step
+        else if (idx == size(p) - 1 
+                        && fabs((p[idx].x - p[idx-1].x) - x_step) < max_diff)
+            ; //nothing, the same step
+        else
+            x_step = 0.;
+    }
+}
+
+namespace {
+vector<int> parse_int_range(string const& s)
+{
+    vector<int> values;
+    vector<string> t = split_string(s, "/");
+    for (vector<string>::const_iterator i = t.begin(); i != t.end(); ++i) {
+        vector<string> a = split_string(*i, "-");
+        if (a.size() == 1 && !a[0].empty()) {
+            int n = strtol(a[0].c_str(), 0, 10);
+            values.push_back(n);
+        }
+        else if (a.size() >= 2 && !a[0].empty() && !a[1].empty()) {
+            int m = strtol(a[0].c_str(), 0, 10);
+            int n = strtol(a[1].c_str(), 0, 10);
+            if (abs(m-n) > 99) // too much..., take only the first one
+                values.push_back(n);
+            else
+                for (int j = min(m,n); j <= max(m,n); ++j)
+                    values.push_back(j);
+        }
+    }
+    return values;
+}
+} //anonymous namespace
+
+vector<string> Data::open_filename_with_columns(string const& file, ifstream& f)
+{
+    vector<string> next_files;
+    // ../data/foo.dat:1,8 -> ../data/foo.dat cols: 1,8
+    // in y column multiple values (2/3/4) or ranges are allowed (foo.dat:1,2-5)
+    string::size_type a = file.find_last_not_of(",0123456789-/");
+    if (a == string::npos || file[a] != ':') 
+        return next_files;
+    string fn(file, 0, a);
+    vector<string> cols_s = split_string(string(file,a+1), ',');
+    if (cols_s.size() < 2 || cols_s.size() > 3)
+        return next_files;
+    vector<int> cols_i;
+    vector<int> yy;
+    for (size_t i = 0; i != cols_s.size(); ++i) {
+        int n;
+        if (cols_s[i].empty())
+            return next_files;
+        if (cols_s[i].find_first_of("/-") != string::npos) {
+            if (i == 1 && isdigit(cols_s[i][0])) { //column y
+                yy = parse_int_range(cols_s[i]);
+                if (yy.empty())
+                    return next_files;
+                n = yy[0];
+                yy.erase(yy.begin());
+            }
+            else // "-" is not accepted in other columns
+                return next_files;
+        }
+        else
+            n = strtol(cols_s[i].c_str(), 0, 10);
+        cols_i.push_back(n);
+    }
+    f.clear();
+    f.open(fn.c_str(), ios::in | ios::binary);
+    if (!f) 
+        throw ExecuteError("Can't open file: " + fn);
+    clear(); //removing previous file
+    filename = fn;   
+    given_cols = cols_i;
+    for (vector<int>::const_iterator i = yy.begin(); i != yy.end(); ++i) {
+        cols_i[1] = *i;
+        next_files.push_back(fn + ":" + join_vector(cols_i, ","));
+    }
+    return next_files;
+}
+
+
+vector<string> Data::load_file (string const& file, string const& type, 
+                                vector<int> const& cols, bool preview)
 {   
-    string const& ft = type.empty() ? guess_file_type(file) // "detect" format
-                                    : type;
+    vector<string> next_files;
     ifstream f (file.c_str(), ios::in | ios::binary);
+    if (f) {
+        clear(); //removing previous file
+        filename = file;   
+        given_cols = cols;
+    }
+    else if (cols.empty()) {
+        next_files = open_filename_with_columns(file, f);
+    }
     if (!f) {
         throw ExecuteError("Can't open file: " + file);
     }
-    clear(); //removing previous file
-    filename = file;   
     given_type = type;
-    given_cols = cols;
 
+    // guess format if not given
+    string const& ft = type.empty() ? guess_file_type(filename) : type;
     if (ft == "text")                         // x y x y ... 
-        load_xy_filetype(f, cols);
+        load_xy_filetype(f, given_cols);
     else if (ft == "htext")                   // header\n x y x y ... 
-        load_header_xy_filetype(f, cols);
+        load_header_xy_filetype(f, given_cols);
     else if (ft == "MCA")                     // .mca
         load_mca_filetype(f);
     else if (ft == "RIT")                     // .rit
@@ -267,13 +423,14 @@ void Data::load_file (string const& file, string const& type,
     }
     if (preview) {
         recompute_y_bounds();
-        return;
+        return next_files;
     }
     if (p.size() < 5)
-        warn ("Only " + S(p.size()) + " data points found in file.");
+        F->warn("Only " + S(p.size()) + " data points found in file.");
     if (!f.eof() && ft != "MCA") //!=mca; .mca doesn't have to reach EOF
-        warn ("Unexpected char when reading " + S (p.size() + 1) + ". point");
+        F->warn("Unexpected char when reading " + S (p.size() + 1) + ". point");
     post_load();
+    return next_files;
 }
 
 
@@ -294,7 +451,7 @@ void Data::load_xy_filetype (ifstream& f, vector<int> const& columns)
     int maxc = *max_element (cols.begin(), cols.end());
     int minc = *min_element (cols.begin(), cols.end());
     if (minc < 0) {
-        warn ("Invalid (negative) column number: " + S(minc));
+        F->warn ("Invalid (negative) column number: " + S(minc));
         return;
     }
     int not_enough_cols = 0, non_data_lines = 0, non_data_blocks = 0;
@@ -327,33 +484,40 @@ void Data::load_xy_filetype (ifstream& f, vector<int> const& columns)
             if (sig > 0) 
                 p.push_back (Point(x, y, sig));
             else
-                warn ("Point " + S(p.size()) + " has sigma = " + S(sig) 
-                        + ". Point canceled.");
+                F->warn ("Point " + S(p.size()) + " has sigma = " + S(sig) 
+                         + ". Point canceled.");
         }
     }
     if (non_data_lines > 0)
-        msg (S(non_data_lines) +" (not empty and not `#...') non-data lines "
+        F->msg(S(non_data_lines) +" (not empty and not `#...') non-data lines "
                 "in " + S(non_data_blocks) + " blocks.");
     if (not_enough_cols > 0)
-        warn ("Less than " + S(maxc) + " numbers in " + S(not_enough_cols) 
-                + " lines.");
+        F->warn("Less than " + S(maxc) + " numbers in " + S(not_enough_cols) 
+                 + " lines.");
     sort(p.begin(), p.end());
     x_step = find_step();
 }
 
-string Data::read_one_line_as_title(ifstream& f, int column)
+/// If the column is non-negative, we try to get n'th word in the line,
+/// where n = column+1 (i.e. column==0 is first word. Otherwise, the line is
+/// returned. 
+/// The hash (#) at the beginning of line is ignored.
+string Data::read_one_line_as_title(ifstream& f, int column/*=-1*/)
 {
     if (!f)
         return "";
     string s;
     getline(f, s);
+    s = strip_string(s);
+    if (!s.empty() && s[0] == '#') 
+        s = string(s, 1);
     --column;
     if (column >= 0) {
         vector<string> v = split_string(s, " \t");
         if (column < size(v)) 
             return v[column];
     }
-    return strip_string(s);
+    return s;
 }
 
 /// read first line as file's title, and than run load_xy_filetype()
@@ -374,7 +538,7 @@ void Data::load_mca_filetype (ifstream& f)
             || *reinterpret_cast<ui2b*>(all_data+34) != 4
             || *reinterpret_cast<ui2b*>(all_data+36) != 2048
             || *reinterpret_cast<ui2b*>(all_data+38) != 1) {
-        warn ("file format different than expected: "+ filename);
+        F->warn ("file format different than expected: "+ filename);
         return;
     }
 
@@ -419,7 +583,7 @@ void Data::load_cpi_filetype (ifstream& f)
     string s;
     getline (f, s);
     if (s.substr(0, header.size()).compare (header) != 0){
-        warn ("No \"" + header + "\" header found.");
+        F->warn ("No \"" + header + "\" header found.");
         return;
     }
     getline (f, s);//xmin
@@ -455,7 +619,7 @@ void Data::load_rit_filetype (ifstream& f)
     vector<fp> num;
     bool r = read_line_and_get_all_numbers(f, num);
     if (!r || num.size() < 2 ){
-        warn ("Bad format in \"header\" of .rit file");
+        F->warn ("Bad format in \"header\" of .rit file");
         return;
     }
     fp xmin = num[0];
@@ -463,8 +627,8 @@ void Data::load_rit_filetype (ifstream& f)
     vector<fp> ys;
     while (read_line_and_get_all_numbers(f, ys)) {
         if (ys.size() == 0)
-            warn ("Error when trying to read " + S (p.size() + 1) + ". point. " 
-                    "Ignoring line.");
+            F->warn("Error when trying to read " + S (p.size() + 1) 
+                     + ". point. Ignoring line.");
         for (vector<fp>::iterator i = ys.begin(); i != ys.end(); i++) {
             fp x = xmin + p.size() * x_step;
             p.push_back (Point(x, *i));
@@ -507,7 +671,7 @@ int Data::auto_range (fp y_level, fp x_margin)
 {
     // pre: p.x sorted
     if (p.empty()) {
-        warn("No points loaded.");
+        F->warn("No points loaded.");
         return -1;
     }
     bool state = (p.begin()->y >= y_level);
@@ -539,7 +703,7 @@ int Data::auto_range (fp y_level, fp x_margin)
 string Data::range_as_string () const 
 {
     if (active_p.empty()) {
-        warn ("File not loaded or all points inactive.");
+        F->warn ("File not loaded or all points inactive.");
         return "[]";
     }
     vector<Point>::const_iterator old_p = p.begin() + active_p[0];
@@ -562,9 +726,11 @@ string Data::range_as_string () const
 ///check for fixed step
 fp Data::find_step() 
 {
-    const fp tiny_relat_diff=0.01;
-    if (p.size() <= 2)
-        return 0;
+    const fp tiny_relat_diff = 1e-4;
+    if (p.size() < 2)
+        return 0.;
+    else if (p.size() == 2)
+        return p[1].x - p[0].x;
     fp min_step, max_step, step;
     min_step = max_step = p[1].x - p[0].x;
     for (vector<Point>::iterator i = p.begin() + 2; i < p.end(); i++) {
@@ -601,7 +767,7 @@ int Data::read_line_and_get_all_numbers(istream &is, vector<fp>& result_numbers)
 int Data::get_lower_bound_ac (fp x) const
 {
     //pre: p.x is sorted, active_p is sorted
-    int pit = lower_bound (p.begin(), p.end(), Point(x)) - p.begin();
+    int pit = lower_bound (p.begin(), p.end(), Point(x,0)) - p.begin();
     return lower_bound (active_p.begin(), active_p.end(), pit) 
         - active_p.begin();
 }
@@ -609,14 +775,14 @@ int Data::get_lower_bound_ac (fp x) const
 int Data::get_upper_bound_ac (fp x) const
 {
     //pre: p.x is sorted, active_p is sorted
-    int pit = upper_bound (p.begin(), p.end(), Point(x)) - p.begin();
+    int pit = upper_bound (p.begin(), p.end(), Point(x,0)) - p.begin();
     return upper_bound (active_p.begin(), active_p.end(), pit) 
         - active_p.begin();
 }
 
 vector<Point>::const_iterator Data::get_point_at(fp x) const
 {
-    return lower_bound (p.begin(), p.end(), Point(x));
+    return lower_bound (p.begin(), p.end(), Point(x,0));
 }
 
 void Data::export_to_file(string filename, vector<string> const& vt, 
@@ -625,7 +791,7 @@ void Data::export_to_file(string filename, vector<string> const& vt,
 {
     ofstream os(filename.c_str(), ios::out | (append ? ios::app : ios::trunc));
     if (!os) {
-        warn ("Can't open file: " + filename);
+        F->warn("Can't open file: " + filename);
         return;
     }
     os << "# " << title << endl;
@@ -660,4 +826,3 @@ void Data::export_to_file(string filename, vector<string> const& vt,
     }
 }
 
-
