@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: data.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: data.h 345 2007-08-21 01:09:15Z wojdyr $
 
 #ifndef FITYK__DATA__H__
 #define FITYK__DATA__H__
@@ -9,24 +9,11 @@
 #include <fstream>
 #include "common.h"
 
+#include "fityk.h" // struct Point
+using fityk::Point;
+using fityk::operator<;
 
-/// data point
-struct Point 
-{
-    fp x, y, sigma;
-    bool is_active;
-    Point () : x(0), y(0), sigma(1), is_active(true) {}
-    Point (fp x_) : x(x_), y(0), sigma(1), is_active(true) {}
-    Point (fp x_, fp y_) : x(x_), y(y_), sigma(1), is_active(true) {}
-    Point (fp x_, fp y_, fp sigma_) : x(x_), y(y_), sigma(sigma_), 
-                                      is_active(true) {}
-    std::string str() { return "(" + S(x) + "; " + S(y) + "; " + S(sigma) 
-                               + (is_active ? ")*" : ") "); }
-};
-
-inline bool operator< (const Point& p, const Point& q) 
-{ return p.x < q.x; }
-    
+class Ftk;
 
 /// dataset
 class Data 
@@ -34,22 +21,27 @@ class Data
 public :
     std::string title;
 
-    Data() : y_min(0.), y_max(1e3) {}
+    Data(Ftk const* F_) : F(F_), y_min(0.), y_max(1e3) {}
     ~Data() {}
-    std::string getInfo() const;
-    void load_file (std::string const& file, std::string const& type, 
-                    std::vector<int> const& cols, bool preview=false);
+    std::string get_info() const;
+
+    ///load data from file, return next filenames if `file' stands for sequance
+    std::vector<std::string> 
+    load_file (std::string const& file, std::string const& type, 
+               std::vector<int> const& cols, bool preview=false);
+
     int load_arrays(std::vector<fp> const& x, std::vector<fp> const& y, 
                    std::vector<fp> const& sigma, std::string const& data_title);
     void load_data_sum(std::vector<Data const*> const& dd, 
                        std::string const& op);
     void clear();
-    void add_point(Point const& pt) { p.push_back(pt); };
+    void add_point(Point const& pt) { p.push_back(pt); }; //don't use it
+    void add_one_point(double x, double y, double sigma);
     static std::string guess_file_type (std::string const& filename);
     fp get_x(int n) const { return p[active_p[n]].x; }
     fp get_y(int n) const { return p[active_p[n]].y; } 
     fp get_sigma (int n) const { return p[active_p[n]].sigma; }
-    int get_n () const { return active_p.size(); }
+    int get_n() const { return active_p.size(); }
     bool is_empty() const { return p.empty(); }
     bool has_any_info() const { return !is_empty() || !get_title().empty(); }
     fp get_x_step() const { return x_step; } /// 0.0 if not fixed
@@ -59,7 +51,7 @@ public :
     std::string range_as_string () const;
     int get_lower_bound_ac (fp x) const; 
     int get_upper_bound_ac (fp x) const;
-    std::string const& get_title() const {return title.empty()?filename:title;}
+    std::string const& get_title() const { return title; }
     std::string const& get_filename() const { return filename; }
 
     void recompute_y_bounds();
@@ -76,11 +68,12 @@ public :
     std::vector<Point> const& points() const { return p; }
     std::string get_given_type() const { return given_type; }
     std::vector<int> get_given_cols() const { return given_cols; }
-    static std::string read_one_line_as_title(std::ifstream& f, int column=0);
+    static std::string read_one_line_as_title(std::ifstream& f, int column=-1);
 private:
+    Ftk const* F;
     std::string filename;
-    std::string given_type;
-    std::vector<int> given_cols;
+    std::string given_type; // filetype explicitely given when loading the file
+    std::vector<int> given_cols; /// columns given when loading the file
     fp x_step; // 0.0 if not fixed;
     bool has_sigma;
     std::vector<Point> p;
@@ -99,6 +92,10 @@ private:
     void load_rit_filetype (std::ifstream& f);
     void load_cpi_filetype (std::ifstream& f);
     void post_load();
+    /// open file with columns embedded into filename (foo.xy:1,2), 
+    // returns next files, if `file' stands for a sequance
+    std::vector<std::string> 
+    open_filename_with_columns(std::string const& file, std::ifstream& f);
 };
 
 #endif
