@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: datatrans.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: datatrans.cpp 288 2007-04-22 02:14:02Z wojdyr $
 
 // the idea of VM is based on one of boost::spirit samples - vmachine_calc
 
@@ -305,21 +305,6 @@ string dt_ops(vector<int> const& code)
 }
 
 
-fp find_idx_in_sorted(vector<Point> const& pp, fp x)
-{
-    if (x <= pp.front().x)
-        return 0;
-    else if (x >= pp.back().x)
-        return pp.size() - 1;
-    vector<Point>::const_iterator i = lower_bound(pp.begin(), pp.end(), x);
-    assert (i > pp.begin() && i < pp.end());
-    if (is_eq(x, i->x))
-            return i - pp.begin();
-    else
-        return i - pp.begin() - (i->x - x) / (i->x - (i-1)->x);
-}
-
-
 //------------------------  Virtual Machine  --------------------------------
 
 vector<int> code;        //  VM code 
@@ -376,6 +361,24 @@ fp get_var_with_idx(fp idx, vector<Point> const& points, T Point::*t)
     }
 }
 
+/// returns floating-point "index" of x in sorted vec of points
+fp find_idx_in_sorted(vector<Point> const& pp, fp x)
+{
+    if (pp.empty())
+        return 0;
+    else if (x <= pp.front().x)
+        return 0;
+    else if (x >= pp.back().x)
+        return pp.size() - 1;
+    vector<Point>::const_iterator i = lower_bound(pp.begin(), pp.end(), 
+                                                  Point(x, 0));
+    assert (i > pp.begin() && i < pp.end());
+    if (is_eq(x, i->x))
+            return i - pp.begin();
+    else
+        return i - pp.begin() - (i->x - x) / (i->x - (i-1)->x);
+}
+
 
 // Return value: 
 //  if once==true (one-time pass).
@@ -1353,7 +1356,7 @@ bool validate_data_expression(string const& str)
 bool is_data_dependent_code(vector<int> const& code)
 {
     for (vector<int>::const_iterator i = code.begin(); i != code.end(); ++i) 
-        if (*i >= OP_VAR_FIRST_OP && *i <= OP_VAR_LAST_OP
+        if ((*i >= OP_VAR_FIRST_OP && *i <= OP_VAR_LAST_OP)
                 || *i == OP_END_AGGREGATE)
             return true;
     return false;
