@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: datatrans.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: datatrans.h 270 2007-03-11 03:08:01Z wojdyr $
 
 #ifndef FITYK__DATATRANS__H__
 #define FITYK__DATATRANS__H__
@@ -9,9 +9,10 @@
 #include <boost/spirit/core.hpp>
 
 #include "common.h"
+#include "data.h" //Point
+
 using namespace boost::spirit;
 
-struct Point;
 class Data;
 
 std::vector<Point> transform_data(std::string const& str, 
