@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: fit.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: fit.h 322 2007-07-24 00:17:11Z wojdyr $
 
 #ifndef FITYK__FIT__H__
 #define FITYK__FIT__H__
@@ -11,6 +11,7 @@
 
 
 class DataWithSum;
+class Ftk;
 
 ///   interface of fitting method and implementation of common functions
 class Fit 
@@ -18,29 +19,32 @@ class Fit
 public:
     std::string const name;
 
-    Fit(std::string m);
+    Fit(Ftk *F_, std::string const& m);
     virtual ~Fit() {};
     void fit(int max_iter, std::vector<DataWithSum*> const& dsds);
     void continue_fit(int max_iter);
-    bool is_initialized(DataWithSum const* ds=0) const
-     { return !ds && !datsums.empty() || datsums.size()==1 && ds==datsums[0]; }
+    bool is_initialized() const { return !datsums.empty(); }
+    bool is_initialized(DataWithSum const* ds) const 
+                          { return datsums.size() == 1 && ds == datsums[0]; }
     bool is_initialized(std::vector<DataWithSum*> const& dsds) const
                                                     { return dsds == datsums; }
-    std::string getInfo(std::vector<DataWithSum*> const& dsds);
+    std::string get_info(std::vector<DataWithSum*> const& dsds);
+    int get_dof(std::vector<DataWithSum*> const& dsds);
     std::string getErrorInfo(std::vector<DataWithSum*> const& dsds, 
                              bool matrix=false);
     std::vector<fp> get_covariance_matrix(std::vector<DataWithSum*> const&dsds);
     std::vector<fp> get_symmetric_errors(std::vector<DataWithSum*> const& dsds);
     std::vector<DataWithSum*> const& get_datsums() const { return datsums; }
     static fp compute_wssr_for_data (DataWithSum const* ds, bool weigthed);
-    static fp do_compute_wssr(std::vector<fp> const &A, 
-                   std::vector<DataWithSum*> const& dsds, bool weigthed);
+    fp do_compute_wssr(std::vector<fp> const &A, 
+                       std::vector<DataWithSum*> const& dsds, bool weigthed);
     static fp compute_r_squared_for_data(DataWithSum const* ds) ;
-    static bool Jordan (std::vector<fp>& A, std::vector<fp>& b, int n); 
-    static void reverse_matrix (std::vector<fp>&A, int n);
-    static std::string print_matrix (const std::vector<fp>& vec, 
+    void Jordan (std::vector<fp>& A, std::vector<fp>& b, int n); 
+    void reverse_matrix (std::vector<fp>&A, int n);
+    std::string print_matrix (const std::vector<fp>& vec, 
                                      int m, int n, char *mname);//m x n
 protected:
+    Ftk *F;
     std::vector<DataWithSum*> datsums;
     int evaluations;
     int max_iterations; //it is set before calling autoiter()
@@ -56,7 +60,8 @@ protected:
     fp compute_wssr(std::vector<fp> const &A, 
                     std::vector<DataWithSum*> const& dsds, bool weigthed=true)
         { ++evaluations; return do_compute_wssr(A, dsds, weigthed); }
-    fp compute_r_squared(std::vector<fp> const &A, std::vector<DataWithSum*> const& dsds) ;
+    fp compute_r_squared(std::vector<fp> const &A, 
+                         std::vector<DataWithSum*> const& dsds) ;
     void compute_derivatives(std::vector<fp> const &A, 
                              std::vector<DataWithSum*> const& dsds, 
                              std::vector<fp>& alpha, std::vector<fp>& beta);
@@ -73,7 +78,7 @@ private:
 class ParameterHistoryMgr
 {
 public:
-    ParameterHistoryMgr() : param_hist_ptr(0) {}
+    ParameterHistoryMgr(Ftk *F_) : F(F_), param_hist_ptr(0) {}
     bool push_param_history(std::vector<fp> const& aa);
     void clear_param_history() { param_history.clear(); param_hist_ptr = 0; }
     int get_param_history_size() const { return param_history.size(); }
@@ -85,32 +90,27 @@ public:
     std::vector<fp> const& get_item(int n) const { return param_history[n]; }
     int get_active_nr() const { return param_hist_ptr; }
 private:
+    Ftk *F;
     std::vector<std::vector<fp> > param_history; /// old parameter vectors
     int param_hist_ptr; /// points to the current/last parameter vector
 };
 
-/// singleton, gives access to fitting methods, enables swithing between them
+/// gives access to fitting methods, enables swithing between them
 /// also stores parameter history
 class FitMethodsContainer : public ParameterHistoryMgr
 {
 public:
-    static FitMethodsContainer* getInstance();
-    int current_method_number() const;
-    Fit const* get_method(int n) const
+    FitMethodsContainer(Ftk *F);
+    ~FitMethodsContainer();
+    Fit* get_method(int n) const
                     { assert(n >= 0 && n<size(methods)); return methods[n]; }
     std::vector<Fit*> const& get_methods() const { return methods; }
-    Fit* getFit() { return methods[current_method_number()]; }
 
 private:
-    static FitMethodsContainer* instance;
     std::vector<Fit*> methods;
 
-    FitMethodsContainer();
     FitMethodsContainer (FitMethodsContainer const&); //disable
-    ~FitMethodsContainer();
 };
 
-inline Fit* getFit() { return FitMethodsContainer::getInstance()->getFit(); }
-
 #endif
 
