@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: func.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: func.cpp 322 2007-07-24 00:17:11Z wojdyr $
 
 #include "common.h"
 #include "func.h"
@@ -8,6 +8,7 @@
 #include "var.h"
 #include "ui.h"
 #include "settings.h"
+#include "logic.h"
 #include "calc.h"
 #include "ast.h"
 
@@ -21,7 +22,9 @@ using namespace boost::spirit;
 std::vector<fp> Function::calc_val_xx(1); 
 std::vector<fp> Function::calc_val_yy(1);
 
-Function::Function (string const &name_, vector<string> const &vars,
+Function::Function (Ftk const* F_,
+                    string const &name_, 
+                    vector<string> const &vars,
                     string const &formula_)
     : VariableUser(name_, "%", vars), 
       type_formula(formula_),
@@ -29,7 +32,7 @@ Function::Function (string const &name_, vector<string> const &vars,
       type_var_names(get_varnames_from_formula(formula_)),
       type_rhs(get_rhs_from_formula(formula_)),
       nv(vars.size()), 
-      settings(getSettings()), 
+      F(F_), 
       center_idx(find_center_in_typevars()),
       vv(vars.size())
 {
@@ -90,14 +93,15 @@ int Function::find_center_in_typevars() const
 }
 
 
-Function* Function::factory (string const &name_, string const &type_name,
+Function* Function::factory (Ftk const* F,
+                             string const &name_, string const &type_name,
                              vector<string> const &vars) 
 {
     string name = name_[0] == '%' ? string(name_, 1) : name_;
 
 #define FACTORY_FUNC(NAME) \
     if (type_name == #NAME) \
-        return new Func##NAME(name, vars);
+        return new Func##NAME(F, name, vars);
 
     FACTORY_FUNC(Constant)
     FACTORY_FUNC(Linear)
@@ -120,9 +124,10 @@ Function* Function::factory (string const &name_, string const &type_name,
     else if (UdfContainer::is_defined(type_name)) {
         UdfContainer::UDF const* udf = UdfContainer::get_udf(type_name);
         if (udf->is_compound)
-            return new CompoundFunction(name, type_name, vars);
+            return new CompoundFunction(F, name, type_name, vars);
         else 
-            return new CustomFunction(name, type_name, vars, udf->op_trees);
+            return new CustomFunction(F, name, type_name, vars, 
+                                      udf->op_trees);
     }
     else 
         throw ExecuteError("Undefined type of function: " + type_name);
@@ -228,7 +233,7 @@ void Function::get_nonzero_idx_range(std::vector<fp> const &xx,
 {
     //precondition: xx is sorted
     fp left, right;
-    bool r = get_nonzero_range(settings->get_cut_level(), left, right);
+    bool r = get_nonzero_range(F->get_settings()->get_cut_level(), left, right);
     if (r) {
         first = lower_bound(xx.begin(), xx.end(), left) - xx.begin(); 
         last = upper_bound(xx.begin(), xx.end(), right) - xx.begin(); 
@@ -261,6 +266,20 @@ void Function::calculate_values_with_params(vector<fp> const& x,
     this_->more_precomputations();
 }
 
+bool Function::has_other_prop(std::string const& name)
+{
+    return contains_element(get_other_prop_names(), name);
+}
+
+std::string Function::other_props_str() const
+{
+    string r;
+    vector<string> v = get_other_prop_names();
+    for (vector<string>::const_iterator i = v.begin(); i != v.end(); ++i)
+        r += (i == v.begin() ? "" : "\n") + *i + ": " + S(other_prop(*i));
+    return r;
+}
+
 string Function::get_info(vector<Variable*> const &variables, 
                           vector<fp> const &parameters, 
                           bool extended) const 
@@ -283,6 +302,8 @@ string Function::get_info(vector<Variable*> const &variables,
         if (this->has_area()) 
             if (!contains_element(type_var_names, string("area")))
                 s += "\nArea: " + S(area());
+        if (this->has_other_props()) 
+            s += "\n" + other_props_str();
     }
     return s;
 } 
@@ -384,7 +405,7 @@ fp Function::find_x_with_value(fp x1, fp x2, fp val, int max_iter) const
 {
     fp y1 = calculate_value(x1) - val;
     fp y2 = calculate_value(x2) - val;
-    if (y1 > 0 && y2 > 0 || y1 < 0 && y2 < 0)
+    if ((y1 > 0 && y2 > 0) || (y1 < 0 && y2 < 0))
         throw ExecuteError("Value " + S(val) + " is not bracketed by "
                            + S(x1) + "(" + S(y1+val) + ") and " 
                            + S(x2) + "(" + S(y2+val) + ").");
@@ -397,7 +418,7 @@ fp Function::find_x_with_value(fp x1, fp x2, fp val, int max_iter) const
     if (y2 == 0)
         return x2;
     if (y1 > 0)
-        Swap(x1, x2);
+        swap(x1, x2);
     fp t = (x1 + x2) / 2.;
     for (int i = 0; i < max_iter; ++i) {
         //check if converged
@@ -422,7 +443,7 @@ fp Function::find_x_with_value(fp x1, fp x2, fp val, int max_iter) const
 
         // select new guess point
         fp dx = -f/df * 1.02; // 1.02 is to jump to the other side of point
-        if (t+dx > x2 && t+dx > x1 || t+dx < x2 && t+dx < x1  // outside
+        if ((t+dx > x2 && t+dx > x1) || (t+dx < x2 && t+dx < x1)  // outside
                             || i % 20 == 19) {                 // precaution
             //bisection
             t = (x1 + x2) / 2.;
@@ -454,7 +475,7 @@ fp Function::find_extremum(fp x1, fp x2, int max_iter) const
     calculate_value_deriv(calc_val_xx, calc_val_yy, dy_da);
     fp y2 = dy_da.back();
 
-    if (y1 > 0 && y2 > 0 || y1 < 0 && y2 < 0)
+    if ((y1 > 0 && y2 > 0) || (y1 < 0 && y2 < 0))
         throw ExecuteError("Derivatives at " + S(x1) + " and " + S(x2) 
                            + " have the same sign.");
     if (y1 == 0)
@@ -462,7 +483,7 @@ fp Function::find_extremum(fp x1, fp x2, int max_iter) const
     if (y2 == 0)
         return x2;
     if (y1 > 0)
-        Swap(x1, x2);
+        swap(x1, x2);
     for (int i = 0; i < max_iter; ++i) {
 
         fp t = (x1 + x2) / 2.;
@@ -686,9 +707,12 @@ vector<string> get_cpd_rhs_components(string const &formula, bool full)
 
 ///////////////////////////////////////////////////////////////////////
 
-CompoundFunction::CompoundFunction(string const &name, string const &type,
+CompoundFunction::CompoundFunction(Ftk const* F,
+                                   string const &name, 
+                                   string const &type,
                                    vector<string> const &vars)
-    : Function(name, vars, get_formula(type)) 
+    : Function(F, name, vars, get_formula(type)),
+      vmgr(F)
 {
     vmgr.silent = true;
     for (int j = 0; j != nv; ++j) 
@@ -767,7 +791,7 @@ bool CompoundFunction::has_center() const
     vector<Function*> const& ff = vmgr.get_functions();
     for (size_t i = 0; i < ff.size(); ++i) {
         if (!ff[i]->has_center()
-                || i > 0 && ff[i-1]->center() != ff[i]->center())
+                || (i > 0 && ff[i-1]->center() != ff[i]->center()))
             return false;
     }
     return true;
@@ -782,7 +806,7 @@ bool CompoundFunction::has_height() const
         return ff[0]->has_center();
     for (size_t i = 0; i < ff.size(); ++i) {
         if (!ff[i]->has_center() || !ff[i]->has_height()
-                || i > 0 && ff[i-1]->center() != ff[i]->center())
+                || (i > 0 && ff[i-1]->center() != ff[i]->center()))
             return false;
     }
     return true;
@@ -839,10 +863,12 @@ bool CompoundFunction::get_nonzero_range(fp level, fp& left, fp& right) const
 
 ///////////////////////////////////////////////////////////////////////
 
-CustomFunction::CustomFunction(string const &name, string const &type,
+CustomFunction::CustomFunction(Ftk const* F,
+                               string const &name, 
+                               string const &type,
                                vector<string> const &vars, 
                                vector<OpTree*> const& op_trees)
-    : Function(name, vars, get_formula(type)),
+    : Function(F, name, vars, get_formula(type)),
       derivatives(nv+1),
       afo(op_trees, value, derivatives) 
 {
