@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: guess.h,v 1.6 2006/09/08 03:35:24 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: guess.h 264 2007-03-01 10:10:54Z wojdyr $
 
 #ifndef FITYK__GUESS__H__
 #define FITYK__GUESS__H__
@@ -29,6 +30,11 @@ void guess_and_add(DataWithSum* ds,
 
 bool is_function_guessable(std::string const& formula, 
                            bool check_defvalue=true);
+
+bool is_function_guessable(std::vector<std::string> const& vars, 
+                           std::vector<std::string> const& defv,
+                           FunctionKind* fk);
+
 bool is_defvalue_guessable(std::string defvalue, FunctionKind k);
 FunctionKind get_function_kind(std::string const& formula);
 
