@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: logic.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: logic.h 322 2007-07-24 00:17:11Z wojdyr $
 
 #ifndef FITYK__LOGIC__H__
 #define FITYK__LOGIC__H__
@@ -9,9 +9,15 @@
 #include <memory>
 #include <algorithm>
 #include "mgr.h"
-//#include "func.h"
+#include "ui.h" //Commands::Status
+
 
 class DataWithSum;
+class Settings;
+class Ftk;
+class UserInterface;
+class FitMethodsContainer;
+class Fit;
 
 /// manages view, i.e. x and y intervals visible currently to the user 
 /// can set view using string like "[20:][-100:1000]" 
@@ -51,7 +57,9 @@ public:
         if (flag&change_top) top=t;
         if (flag&change_bottom) bottom=b;
     }
+    /// set datasets that are to be used in fit()
     void set_datasets(std::vector<DataWithSum*> const& dd);
+    /// fit specified edges to the data range 
     void fit(int flag=fit_all); 
     void parse_and_set(std::vector<std::string> const& lrbt); 
 protected:
@@ -66,7 +74,7 @@ protected:
 class DataWithSum
 {
 public:
-    DataWithSum(VariableManager *mgr, Data* data_=0);
+    DataWithSum(Ftk *F, Data* data_=0);
     Data *get_data() const { return data.get(); } 
     Sum *get_sum() const { return sum.get(); }
     bool has_any_info() const;
@@ -80,39 +88,80 @@ private:
 
 
 /// keeps all functions, variables, parameters, datasets with sums and View
-class ApplicationLogic : public VariableManager
+class Ftk : public VariableManager
 {
 public:
     View view;
     /// used for randomly drawing parameter values, in fitting methods like GA
     fp default_relative_domain_width;
 
-    ApplicationLogic() : default_relative_domain_width(0.1) { start_app(); }
-
-    ~ApplicationLogic() { stop_app(); }
-    void start_app();
-    void stop_app();
+    Ftk();
+    ~Ftk();
+    /// reset everything but UserInterface (and related settings)
+    void reset();
     void dump_all_as_script (std::string const &filename);
 
     void activate_ds(int d);
     int append_ds(Data *data=0);
     void remove_ds(int d);
     int get_ds_count() const { return dsds.size(); }
-    DataWithSum* get_ds(int n);
+    DataWithSum* get_ds(int n) { return dsds[check_ds_number(n)]; }
+    DataWithSum const* get_ds(int n) const { return dsds[check_ds_number(n)]; }
     std::vector<DataWithSum*> const& get_dsds() const { return dsds; }
     int get_active_ds_position() const { return active_ds; }
     Data *get_data(int n) { return get_ds(n)->get_data(); }
-    Sum *get_sum(int n)   { return get_ds(n)->get_sum();  }
+    Sum *get_sum(int n)   { return get_ds(n)->get_sum(); }
     bool has_ds(DataWithSum const* p) const 
                       { return count(dsds.begin(), dsds.end(), p) > 0; }
-    const Function* find_function_any(std::string const &fstr);
+    std::string find_function_name(std::string const &fstr) const;
+    const Function* find_function_any(std::string const &fstr) const;
+
+    Settings const* get_settings() const { return settings; }
+    Settings* get_settings() { return settings; }
+
+    UserInterface const* get_ui() const { return ui; }
+    UserInterface* get_ui() { return ui; }
+
+    FitMethodsContainer const* get_fit_container() const {return fit_container;}
+    FitMethodsContainer* get_fit_container() { return fit_container; }
+    Fit* get_fit(); 
+
+    /// Send warning to UI. 
+    void warn(std::string const &s) const;
+
+    /// Send implicitely requested message to UI. 
+    void rmsg(std::string const &s) const;
+
+    /// Send message to UI. 
+    void msg(std::string const &s) const; 
+
+    /// Send verbose message to UI. 
+    void vmsg(std::string const &s) const;
+
+    int get_verbosity() const;
+
+    /// execute command(s) from string
+    Commands::Status exec(std::string const &s);
+
+    /// import dataset (or multiple datasets, in special cases)
+    void import_dataset(int slot, std::string const& filename, 
+                        std::string const& type, std::vector<int> const& cols);
 
 protected:
     std::vector<DataWithSum*> dsds;
     int active_ds;
+    Settings* settings;
+    UserInterface* ui;
+    FitMethodsContainer* fit_container;
+
+    void initialize();
+    void destroy();
+
+private:
+    /// verify that n is the valid number for get_ds() and return n 
+    int check_ds_number(int n) const;
 };
 
-
-extern ApplicationLogic *AL;
+extern Ftk* AL;
 
 #endif 
