@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: sum.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: sum.h 322 2007-07-24 00:17:11Z wojdyr $
 
 #ifndef FITYK__SUM__H__
 #define FITYK__SUM__H__
@@ -13,6 +13,7 @@
 class Function;
 class Data;
 class VariableManager;
+class Ftk;
 
 ///  This class contains description of curve which we are trying to fit 
 ///  to data. This curve is described simply by listing names of functions
@@ -20,7 +21,7 @@ class VariableManager;
 class Sum 
 {
 public:
-    Sum(VariableManager *mgr_);
+    Sum(Ftk *F_);
     ~Sum();
     void find_function_indices();
     void add_function_to(std::string const &name, char add_to);
@@ -56,6 +57,7 @@ public:
     bool is_dependent_on_var(int idx) const;
 
 private:
+    Ftk const* F;
     VariableManager &mgr;
     std::vector<std::string> ff_names;
     std::vector<std::string> zz_names;
