@@ -1,11 +1,10 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: ui.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: ui.cpp 322 2007-07-24 00:17:11Z wojdyr $
 
 #include "common.h"
 #include "ui.h"
 #include "settings.h"
-#include <fstream>
 #include <string>
 #include <iostream>
 #include "logic.h"
@@ -38,7 +37,7 @@ void Commands::put_command(string const &c, Commands::Status r)
         log << " " << c << endl; 
 }
 
-void Commands::put_output_message(string const& s)
+void Commands::put_output_message(string const& s) const
 {
     if (!log_filename.empty() && log_with_output) {
         // insert "# " at the beginning of string and before every new line
@@ -105,7 +104,8 @@ string Commands::get_info(bool extended) const
 }
 
 
-void Commands::start_logging(string const& filename, bool with_output)
+void Commands::start_logging(string const& filename, bool with_output,
+                             Ftk const* F)
 {
     if (filename.empty())
        stop_logging(); 
@@ -127,11 +127,11 @@ void Commands::start_logging(string const& filename, bool with_output)
         log_with_output = false; //don't put info() below into log 
         if (with_output) {
             log << "INPUT AND OUTPUT";
-            msg ("Logging input and output to file: " + filename);
+            F->msg("Logging input and output to file: " + filename);
         }
         else {
             log << "INPUT";
-            msg ("Logging input to file: " + filename);
+            F->msg("Logging input to file: " + filename);
         }
         log << " TO THIS FILE (" << filename << ")\n";
         log_with_output = with_output;
@@ -154,44 +154,33 @@ struct NumberedLine
     NumberedLine(int nr_, string txt_) : nr(nr_), txt(txt_) {}
 };
 
-
-//this is a part of Singleton design pattern
-UserInterface* UserInterface::instance = 0; 
-UserInterface* UserInterface::getInstance() 
+Commands::Status UserInterface::exec_and_log(string const &c)
 {
-    if (instance == 0)  // is it the first call?
-        instance = new UserInterface; // create sole instance
-    return instance; // address of sole instance
-}
-
-Commands::Status UserInterface::execAndLogCmd(string const &c)
-{
-    Commands::Status r = execCommand(c); 
+    Commands::Status r = this->exec_command(c); 
     commands.put_command(c, r); 
     return r;
 }
 
-void UserInterface::outputMessage (OutputStyle style, const string& s)
+void UserInterface::output_message (OutputStyle style, const string& s) const
 {
     if (keep_quiet)
         return;
-    showMessage(style, s);
+    show_message(style, s);
     commands.put_output_message(s);
-    if (style == os_warn && getSettings()->get_b("exit-on-warning")) {
-        showMessage(os_normal, "Warning -> exiting program.");
+    if (style == os_warn && F->get_settings()->get_b("exit-on-warning")) {
+        show_message(os_normal, "Warning -> exiting program.");
         throw ExitRequestedException();
     }
 }
 
-
 /// items in selected_lines are ranges (first, after-last).
 /// if selected_lines are empty - all lines from file are executed
-void UserInterface::execScript (const string& filename, 
+void UserInterface::exec_script(const string& filename, 
                                 const vector<pair<int,int> >& selected_lines)
 {
-    ifstream file (filename.c_str(), ios::in);
+    ifstream file(filename.c_str(), ios::in);
     if (!file) {
-        warn ("Can't open file: " + filename);
+        F->warn("Can't open file: " + filename);
         return;
     }
 
@@ -199,10 +188,13 @@ void UserInterface::execScript (const string& filename,
                          exec_nls; //lines to execute (in order of execution)
 
     //fill nls for easier manipulation of file lines
+    string dir = get_directory(filename);
     string s;
     int line_index = 0;
-    while (getline (file, s)) 
+    while (getline (file, s)) {
+        replace_all(s, "_EXECUTED_SCRIPT_DIR_/", dir);
         nls.push_back(NumberedLine(++line_index, s));
+    }
 
     if (selected_lines.empty())
         exec_nls = nls;
@@ -218,24 +210,30 @@ void UserInterface::execScript (const string& filename,
                                                     i != exec_nls.end(); i++) {
         if (i->txt.length() == 0)
             continue;
-        if (getVerbosity() >= 0)
-            showMessage (os_quot, S(i->nr) + "> " + i->txt); 
+        if (get_verbosity() >= 0)
+            show_message (os_quot, S(i->nr) + "> " + i->txt); 
         // result of parse_and_execute here is neglected. Errors in script
         // don't change status of command, which executes script
         parse_and_execute(i->txt);
     }
 }
 
-void UserInterface::drawPlot (int pri, bool now)
+void UserInterface::draw_plot (int pri, bool now)
 {
-    if (pri <= getSettings()->get_e("autoplot")) 
-        doDrawPlot(now);
+    if (pri <= F->get_settings()->get_e("autoplot")) 
+        do_draw_plot(now);
 }
 
 
-int UserInterface::getVerbosity() { return getSettings()->get_e("verbosity"); }
+int UserInterface::get_verbosity() const
+                           { return F->get_settings()->get_e("verbosity"); }
 
 
+Commands::Status UserInterface::exec_command (std::string const &s)
+{ 
+    return m_exec_command ? (*m_exec_command)(s) : parse_and_execute(s); 
+}
+
 bool is_fityk_script(string filename)
 {
     const char *magic = "# Fityk";
@@ -257,10 +255,12 @@ bool is_fityk_script(string filename)
 
 void UserInterface::process_cmd_line_filename(string const& par)
 {
-    if (is_fityk_script(par))
-        getUI()->execScript(par);
+    if (startswith(par, "=->"))
+        exec_and_log(string(par, 3));
+    else if (is_fityk_script(par))
+        exec_script(par);
     else {
-        getUI()->execAndLogCmd("@+ <'" + par + "'");
+        exec_and_log("@+ <'" + par + "'");
     }
 }
 
