@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: dataedit.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: dataedit.cpp 317 2007-07-15 00:39:46Z wojdyr $
 
 /// In this file:
 ///  Data Editor (DataEditorDlg) and helpers
@@ -18,7 +18,7 @@
 #include "gui.h"
 #include "dialogs.h" //export_data_dlg()
 #include "../data.h" // Data, Point
-#include "../ui.h"  //exec_command()
+#include "../logic.h"  
 #include "../datatrans.h" //validate_transformation()
 
 using namespace std;
@@ -40,7 +40,7 @@ public:
                                             data(data_), ded(ded_) {}
     int GetNumberRows() { return data->points().size(); }
     int GetNumberCols() { return 4; }
-    bool IsEmptyCell(int WXUNUSED(row), int WXUNUSED(col)) { return false; }
+    bool IsEmptyCell(int /*row*/, int /*col*/) { return false; }
 
     wxString GetValue(int row, int col) 
     {  
@@ -52,7 +52,7 @@ public:
 
     void SetValue(int, int, const wxString&) { assert(0); }
 
-    wxString GetTypeName(int WXUNUSED(row), int col)
+    wxString GetTypeName(int /*row*/, int col)
         { return col == 0 ? wxGRID_VALUE_BOOL : wxGRID_VALUE_FLOAT; }
 
     bool CanGetValueAs(int row, int col, const wxString& typeName)
@@ -81,8 +81,7 @@ public:
             case 3: t = "S";  break;
             default: assert(0);
         }
-        exec_command(t + "[" + S(row)+"]=" + S(value) 
-                                                  + frame->get_in_dataset());
+        ftk->exec(t + "[" + S(row)+"]=" + S(value) + frame->get_in_dataset());
         if (col == 1) // order of items can be changed
             ded->grid->ForceRefresh();
         ded->rezoom_btn->Enable();
@@ -91,7 +90,7 @@ public:
     void SetValueAsBool(int row, int col, bool value) 
     { 
         assert(col==0); 
-        exec_command("A[" + S(row)+"]=" + (value?"true":"false") 
+        ftk->exec("A[" + S(row)+"]=" + (value?"true":"false") 
                                                   + frame->get_in_dataset()); 
         ded->rezoom_btn->Enable();
     }
@@ -227,7 +226,7 @@ DataEditorDlg::DataEditorDlg (wxWindow* parent, wxWindowID id,
     wxBoxSizer *left_sizer = new wxBoxSizer(wxVERTICAL);
     left_sizer->Add(new wxStaticText(left_panel, -1,wxT("Original filename:")));
     filename_label = new wxStaticText(left_panel, -1, wxT(""));
-    left_sizer->Add(filename_label, 0, wxADJUST_MINSIZE);
+    left_sizer->Add(filename_label, 0);
     wxBoxSizer *two_btn_sizer = new wxBoxSizer(wxHORIZONTAL);
     revert_btn = new wxButton(left_panel, wxID_REVERT_TO_SAVED, 
                               wxT("Revert to Saved"));
@@ -239,7 +238,7 @@ DataEditorDlg::DataEditorDlg (wxWindow* parent, wxWindowID id,
     left_sizer->Add(new wxStaticText(left_panel, -1, wxT("Data title: ")),
                     0, wxLEFT|wxRIGHT|wxTOP, 5);
     title_label = new wxStaticText(left_panel, -1, wxT(""));
-    left_sizer->Add(title_label, 0, wxLEFT|wxRIGHT|wxBOTTOM|wxADJUST_MINSIZE,5);
+    left_sizer->Add(title_label, 0, wxLEFT|wxRIGHT|wxBOTTOM, 5);
     grid = new wxGrid(left_panel, ID_DE_GRID, 
                       wxDefaultPosition, wxSize(-1, 350));
     left_sizer->Add(grid, 1, wxEXPAND);
@@ -273,7 +272,7 @@ DataEditorDlg::DataEditorDlg (wxWindow* parent, wxWindowID id,
     description = new wxStaticText(right_panel, -1, wxT("\n\n\n\n"), 
                                    wxDefaultPosition, wxDefaultSize,
                                    wxALIGN_LEFT);
-    right_sizer->Add(description, 0, wxEXPAND|wxALL|wxADJUST_MINSIZE, 5);
+    right_sizer->Add(description, 0, wxEXPAND|wxALL, 5);
     code = new wxTextCtrl(right_panel, ID_DE_CODE, wxT(""), 
                           wxDefaultPosition, wxDefaultSize,
                           wxTE_MULTILINE|wxHSCROLL|wxVSCROLL);
@@ -401,7 +400,6 @@ void DataEditorDlg::update_data(ndnd_type const& dd)
     filename_label->SetLabel(s2wx(filename));
     revert_btn->Enable(!filename.empty());
     title_label->SetLabel(s2wx(title));
-    Show();
 }
 
 void DataEditorDlg::refresh_grid()
@@ -416,7 +414,7 @@ void DataEditorDlg::refresh_grid()
     grid->AdjustScrollbars();
 }
 
-void DataEditorDlg::OnRevert (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnRevert (wxCommandEvent&)
 {
     string cmd;
     for (ndnd_type::const_iterator i = ndnd.begin(); i != ndnd.end(); ++i) {
@@ -428,11 +426,11 @@ void DataEditorDlg::OnRevert (wxCommandEvent& WXUNUSED(event))
     }
     if (cmd.empty())
         return;
-    exec_command(cmd);
+    ftk->exec(cmd);
     refresh_grid();
 }
 
-void DataEditorDlg::OnSaveAs (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnSaveAs (wxCommandEvent&)
 {
     if (ndnd.size() != 1)
         return;
@@ -442,7 +440,7 @@ void DataEditorDlg::OnSaveAs (wxCommandEvent& WXUNUSED(event))
     }
 }
 
-void DataEditorDlg::OnAdd (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnAdd (wxCommandEvent&)
 {
     DataTransform new_transform("new", "useful",
                                  "", wx2s(code->GetValue()));
@@ -452,10 +450,13 @@ void DataEditorDlg::OnAdd (wxCommandEvent& WXUNUSED(event))
         transforms.insert(transforms.begin() + pos, new_transform);
         insert_trans_list_item(pos);
         select_transform(pos);
+
+        FFrame *fframe = static_cast<FFrame *>(GetParent());
+        fframe->update_menu_fast_tranforms();
     }
 }
 
-void DataEditorDlg::OnRemove (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnRemove (wxCommandEvent&)
 {
     int item = get_selected_item();
     if (item == -1 || transforms[item].category == "builtin")
@@ -465,7 +466,7 @@ void DataEditorDlg::OnRemove (wxCommandEvent& WXUNUSED(event))
     select_transform(item > 0 ? item-1 : 0);
 }
 
-void DataEditorDlg::OnUp (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnUp (wxCommandEvent&)
 {
     int item = get_selected_item();
     if (item == 0)
@@ -480,7 +481,7 @@ void DataEditorDlg::OnUp (wxCommandEvent& WXUNUSED(event))
     down_btn->Enable(true);
 }
 
-void DataEditorDlg::OnDown (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnDown (wxCommandEvent&)
 {
     int item = get_selected_item();
     if (item >= size(transforms) - 1)
@@ -495,7 +496,7 @@ void DataEditorDlg::OnDown (wxCommandEvent& WXUNUSED(event))
     down_btn->Enable(item+1 < trans_list->GetItemCount() - 1);
 }
 
-void DataEditorDlg::OnSave (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnSave (wxCommandEvent&)
 {
     wxString transform_path = get_user_conffile("transform");
     ofstream f(wx2s(transform_path).c_str());
@@ -505,19 +506,19 @@ void DataEditorDlg::OnSave (wxCommandEvent& WXUNUSED(event))
             f << i->as_fileline() << endl;
 }
 
-void DataEditorDlg::OnReset (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnReset (wxCommandEvent&)
 {
     initialize_transforms(true);
 }
 
-void DataEditorDlg::OnApply (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnApply (wxCommandEvent&)
 {
     execute_tranform(wx2s(code->GetValue().Trim()));
     refresh_grid();
     rezoom_btn->Enable();
 }
 
-void DataEditorDlg::OnReZoom (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnReZoom (wxCommandEvent&)
 {
     frame->GViewAll();
     rezoom_btn->Enable(false);
@@ -534,10 +535,10 @@ void DataEditorDlg::execute_tranform(string code)
         if (i+1 != cmds.end())
             t += ";";
     }
-    exec_command(t);
+    ftk->exec(t);
 }
 
-void DataEditorDlg::OnHelp (wxCommandEvent& WXUNUSED(event))
+void DataEditorDlg::OnHelp (wxCommandEvent&)
 {
     frame->display_help_section("Data transformations");
 }
@@ -553,7 +554,8 @@ void DataEditorDlg::CodeText()
     wxString text = code->GetValue().Trim();
     if (check_syntax) {
         string text = wx2s(code->GetValue());
-        replace_all(text, "\n", " & ");
+        replace_all(text, "\n", " , ");
+        // 'text' is not identical with the final command (, instead of ;)
         apply_btn->Enable(validate_transformation(text));
     }
     else
