@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: sidebar.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: sidebar.cpp 345 2007-08-21 01:09:15Z wojdyr $
 
 /// In this file:
 ///  SideBar class, the right hand sidebar in GUI 
@@ -141,7 +141,6 @@ SideBar::SideBar(wxWindow *parent, wxWindowID id)
     choices.Add(wxT("hide all"));
     data_look = new wxChoice(data_page, ID_DP_LOOK,
                              wxDefaultPosition, wxDefaultSize, choices);
-    data_look->Select(0);
     data_look_sizer->Add(data_look, 1, wxEXPAND);
     data_sizer->Add(data_look_sizer, 0, wxEXPAND);
 
@@ -263,20 +262,22 @@ SideBar::SideBar(wxWindow *parent, wxWindowID id)
     bottom_panel->SetAutoLayout(true);
 }
 
-void SideBar::OnDataButtonNew (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnDataButtonNew (wxCommandEvent&)
 {
-    exec_command("@+");
+    ftk->exec("@+");
+    frame->refresh_plots(false, true);
 }
 
-void SideBar::OnDataButtonDup (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnDataButtonDup (wxCommandEvent&)
 {
-    exec_command("@+ = " + join_vector(d->get_selected_data(), " + "));
+    ftk->exec("@+ = sum_same_x " + join_vector(d->get_selected_data(), " + "));
+    frame->refresh_plots(false, true);
 }
 
-void SideBar::OnDataButtonRen (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnDataButtonRen (wxCommandEvent&)
 {
     int n = get_focused_data();
-    Data *data = AL->get_data(n);
+    Data *data = ftk->get_data(n);
     wxString old_title = s2wx(data->get_title());
 
     wxString s = wxGetTextFromUser(
@@ -284,7 +285,7 @@ void SideBar::OnDataButtonRen (wxCommandEvent& WXUNUSED(event))
                          wxT("Rename dataset"),
                          old_title);
     if (!s.IsEmpty() && s != old_title)
-        exec_command("@" + S(n) + ".title = '" + wx2s(s) + "'");
+        ftk->exec("@" + S(n) + ".title = '" + wx2s(s) + "'");
 }
 
 void SideBar::delete_selected_items()
@@ -293,33 +294,35 @@ void SideBar::delete_selected_items()
     if (n < 0)
         return;
     string txt = wx2s(nb->GetPageText(n));
-    if (txt == "data")
-        exec_command("delete " + join_vector(d->get_selected_data(), ", "));
+    if (txt == "data") {
+        ftk->exec("delete " + join_vector(d->get_selected_data(), ", "));
+        frame->refresh_plots(false, true);
+    }
     else if (txt == "functions")
-        exec_command("delete " + join_vector(get_selected_func(), ", "));
+        ftk->exec("delete " + join_vector(get_selected_func(), ", "));
     else if (txt == "variables")
-        exec_command("delete " + join_vector(get_selected_vars(), ", "));
+        ftk->exec("delete " + join_vector(get_selected_vars(), ", "));
     else
         assert(0);
 }
 
-void SideBar::OnDataButtonCopyF (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnDataButtonCopyF (wxCommandEvent&)
 {
     int n = get_focused_data();
-    if (n+1 >= AL->get_ds_count())
+    if (n+1 >= ftk->get_ds_count())
         return;
     d->list->Select(n, false);
     d->list->Select(n+1, true);
     d->list->Focus(n+1);
     DataFocusChanged();
     string cmd = "@" + S(n+1) + ".F=copy(@" + S(n) + ".F)";
-    if (!AL->get_sum(n)->get_zz_names().empty() 
-            || !AL->get_sum(n+1)->get_zz_names().empty())
+    if (!ftk->get_sum(n)->get_zz_names().empty() 
+            || !ftk->get_sum(n+1)->get_zz_names().empty())
         cmd += "; @" + S(n+1) + ".Z=copy(@" + S(n) + ".Z)";
-    exec_command(cmd);
+    ftk->exec(cmd);
 }
 
-void SideBar::OnDataButtonCol (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnDataButtonCol (wxCommandEvent&)
 {
     int sel_size = d->list->GetSelectedItemCount();
     if (sel_size == 0)
@@ -365,12 +368,12 @@ void SideBar::OnDataColorsChanged(GradientDlg *gd)
 }
 
 
-void SideBar::OnDataLookChanged (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnDataLookChanged (wxCommandEvent&)
 {
     frame->refresh_plots(false, true);
 }
 
-void SideBar::OnDataShiftUpChanged (wxSpinEvent& WXUNUSED(event))
+void SideBar::OnDataShiftUpChanged (wxSpinEvent&)
 {
     frame->refresh_plots(false, true);
 }
@@ -397,12 +400,12 @@ void SideBar::OnDataPSChanged (wxCommandEvent& event)
     frame->refresh_plots(false, true);
 }
 
-void SideBar::OnFuncFilterChanged (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnFuncFilterChanged (wxCommandEvent&)
 {
     update_lists(false);
 }
 
-void SideBar::OnFuncButtonNew (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnFuncButtonNew (wxCommandEvent&)
 {
     string peak_type = frame->get_peak_type();
     string formula = Function::get_formula(peak_type);
@@ -412,24 +415,24 @@ void SideBar::OnFuncButtonNew (wxCommandEvent& WXUNUSED(event))
     frame->edit_in_input(t);
 }
 
-void SideBar::OnFuncButtonEdit (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnFuncButtonEdit (wxCommandEvent&)
 {
     if (!bp_func)
         return;
-    string t = bp_func->get_current_assignment(AL->get_variables(), 
-                                               AL->get_parameters());
+    string t = bp_func->get_current_assignment(ftk->get_variables(), 
+                                               ftk->get_parameters());
     frame->edit_in_input(t);
 }
 
-void SideBar::OnFuncButtonChType (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnFuncButtonChType (wxCommandEvent&)
 {
-    warn("Sorry. Changing type of function is not implemented yet.");
+    ftk->warn("Sorry. Changing type of function is not implemented yet.");
 }
 
-void SideBar::OnFuncButtonCol (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnFuncButtonCol (wxCommandEvent&)
 {
     vector<int> const& ffi 
-        = AL->get_sum(AL->get_active_ds_position())->get_ff_idx();
+        = ftk->get_sum(ftk->get_active_ds_position())->get_ff_idx();
     vector<int>::const_iterator in_ff = find(ffi.begin(), ffi.end(), 
                                                          active_function);
     if (in_ff == ffi.end())
@@ -443,18 +446,18 @@ void SideBar::OnFuncButtonCol (wxCommandEvent& WXUNUSED(event))
     }
 }
 
-void SideBar::OnVarButtonNew (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnVarButtonNew (wxCommandEvent&)
 {
     frame->edit_in_input("$put_name_here = ");
 }
 
-void SideBar::OnVarButtonEdit (wxCommandEvent& WXUNUSED(event))
+void SideBar::OnVarButtonEdit (wxCommandEvent&)
 {
     int n = get_focused_var();
-    if (n < 0 || n >= size(AL->get_variables()))
+    if (n < 0 || n >= size(ftk->get_variables()))
         return;
-    Variable const* var = AL->get_variable(n);
-    string t = var->xname + " = "+ var->get_formula(AL->get_parameters());
+    Variable const* var = ftk->get_variable(n);
+    string t = var->xname + " = "+ var->get_formula(ftk->get_parameters());
     frame->edit_in_input(t);
 }
 
@@ -464,11 +467,16 @@ void SideBar::read_settings(wxConfigBase *cf)
     d->split(cfg_read_double(cf, wxT("dataProportion"), 0.75));
     f->split(cfg_read_double(cf, wxT("funcProportion"), 0.75));
     v->split(cfg_read_double(cf, wxT("varProportion"), 0.75));
+    data_look->Select(cf->Read(wxT("dataLook"), 0L));
 }
 
 void SideBar::save_settings(wxConfigBase *cf) const
 {
     cf->SetPath(wxT("/SideBar"));
+    cf->Write(wxT("dataProportion"), d->GetProportion());
+    cf->Write(wxT("funcProportion"), f->GetProportion());
+    cf->Write(wxT("varProportion"), v->GetProportion());
+    cf->Write(wxT("dataLook"), data_look->GetSelection());
 }
 
 void SideBar::update_lists(bool nondata_changed)
@@ -479,7 +487,7 @@ void SideBar::update_lists(bool nondata_changed)
     update_var_list();
     
     //-- enable/disable buttons
-    bool not_the_last = get_focused_data()+1 < AL->get_ds_count();
+    bool not_the_last = get_focused_data()+1 < ftk->get_ds_count();
     data_page->FindWindow(ID_DP_CPF)->Enable(not_the_last);
     bool has_any_funcs = (f->list->GetItemCount() > 0);
     func_page->FindWindow(ID_FP_DEL)->Enable(has_any_funcs);
@@ -508,9 +516,9 @@ void SideBar::update_func_list(bool nondata_changed)
     wxColour const& bg_col = mplot->get_bg_color();
 
     //functions filter
-    while ((int) filter_ch->GetCount() > AL->get_ds_count() + 1)
+    while ((int) filter_ch->GetCount() > ftk->get_ds_count() + 1)
         filter_ch->Delete(filter_ch->GetCount()-1);
-    for (int i = filter_ch->GetCount() - 1; i < AL->get_ds_count(); ++i)
+    for (int i = filter_ch->GetCount() - 1; i < ftk->get_ds_count(); ++i)
         filter_ch->Append(wxString::Format(wxT("only functions from @%i"), i));
 
     //functions
@@ -518,23 +526,23 @@ void SideBar::update_func_list(bool nondata_changed)
     static int old_func_size;
     vector<string> func_data;
     vector<int> new_func_col_id;
-    int active_ds_pos = AL->get_active_ds_position();
-    Sum const* sum = AL->get_sum(active_ds_pos);
+    int active_ds_pos = ftk->get_active_ds_position();
+    Sum const* sum = ftk->get_sum(active_ds_pos);
     Sum const* filter_sum = 0;
     if (filter_ch->GetSelection() > 0)
-        filter_sum = AL->get_sum(filter_ch->GetSelection()-1);
-    int func_size = AL->get_functions().size();
+        filter_sum = ftk->get_sum(filter_ch->GetSelection()-1);
+    int func_size = ftk->get_functions().size();
     if (active_function == -1)
         active_function = func_size - 1;
     else {
         if (active_function >= func_size || 
-                AL->get_function(active_function) != bp_func)
-            active_function = AL->find_function_nr(active_function_name);
+                ftk->get_function(active_function) != bp_func)
+            active_function = ftk->find_function_nr(active_function_name);
         if (active_function == -1 || func_size == old_func_size+1)
             active_function = func_size - 1;
     }
     if (active_function != -1)
-        active_function_name = AL->get_function(active_function)->name;
+        active_function_name = ftk->get_function(active_function)->name;
     else
         active_function_name = "";
 
@@ -545,7 +553,7 @@ void SideBar::update_func_list(bool nondata_changed)
             continue;
         if (i == active_function)
             pos = new_func_col_id.size();
-        Function const* f = AL->get_function(i);
+        Function const* f = ftk->get_function(i);
         func_data.push_back(f->name);
         func_data.push_back(f->type_name);
         func_data.push_back(f->has_center() ? S(f->center()).c_str() : "-");
@@ -585,15 +593,15 @@ void SideBar::update_var_list()
 {
     vector<string> var_data;
     //  count references first
-    vector<Variable*> const& variables = AL->get_variables();
+    vector<Variable*> const& variables = ftk->get_variables();
     vector<int> var_vrefs(variables.size(), 0), var_frefs(variables.size(), 0);
     for (vector<Variable*>::const_iterator i = variables.begin();
                                                 i != variables.end(); ++i) {
         for (int j = 0; j != (*i)->get_vars_count(); ++j)
             var_vrefs[(*i)->get_var_idx(j)]++;
     }
-    for (vector<Function*>::const_iterator i = AL->get_functions().begin();
-                                         i != AL->get_functions().end(); ++i) {
+    for (vector<Function*>::const_iterator i = ftk->get_functions().begin();
+                                         i != ftk->get_functions().end(); ++i) {
         for (int j = 0; j != (*i)->get_vars_count(); ++j)
             var_frefs[(*i)->get_var_idx(j)]++;
     }
@@ -605,7 +613,7 @@ void SideBar::update_var_list()
                       + S(v->get_vars_count());
         var_data.push_back(refs); //refs
         var_data.push_back(S(v->get_value())); //value
-        var_data.push_back(v->get_formula(AL->get_parameters()));  //formula
+        var_data.push_back(v->get_formula(ftk->get_parameters()));  //formula
     }
     v->list->populate(var_data);
 }
@@ -618,7 +626,7 @@ int SideBar::get_focused_data() const
 
 int SideBar::get_focused_var() const 
 { 
-    if (AL->get_variables().empty())
+    if (ftk->get_variables().empty())
         return -1;
     else {
         int n = v->list->GetFocusedItem(); 
@@ -626,10 +634,11 @@ int SideBar::get_focused_var() const
     }
 }
 
-bool SideBar::is_func_selected(int n) const 
-{ 
-    return f->list->IsSelected(n) || f->list->GetFocusedItem() == n; 
-}
+//bool SideBar::is_func_selected(int n) const 
+//{ 
+//    return f->list->IsSelected(n) || f->list->GetFocusedItem() == n; 
+//}
+//
 
 void SideBar::activate_function(int n)
 {
@@ -646,7 +655,7 @@ void SideBar::activate_function(int n)
 void SideBar::do_activate_function()
 {
     if (active_function != -1)
-        active_function_name = AL->get_function(active_function)->name;
+        active_function_name = ftk->get_function(active_function)->name;
     else
         active_function_name = "";
     frame->refresh_plots(false, true);
@@ -659,18 +668,26 @@ void SideBar::DataFocusChanged()
     int n = d->list->GetFocusedItem();
     if (n < 0)
         return;
-    int length = AL->get_ds_count();
-    if (length > 1 && AL->get_active_ds_position() != n) 
-        exec_command("plot .. in @" + S(n));
+    int length = ftk->get_ds_count();
+    if (length > 1 && ftk->get_active_ds_position() != n) 
+        ftk->exec("plot .. in @" + S(n));
     data_page->FindWindow(ID_DP_CPF)->Enable(n+1<length);
 }
 
 string SideBar::get_datasets_str()
 {
-    if (data_look->GetSelection() == 0)
+    if (data_look->GetSelection() == 0) // all datasets
         return "@*";
-    else
-        return frame->get_active_data_str();
+    else { // only selected datasets
+        int first = d->list->GetFirstSelected();
+        if (first == -1)
+            return frame->get_active_data_str();
+        string s = "@" + S(first);
+        for (int i = d->list->GetNextSelected(first); i != -1; 
+                                              i = d->list->GetNextSelected(i))
+            s += ", @" + S(i);
+        return s;
+    }
 }
 
 void SideBar::update_data_inf()
@@ -689,7 +706,10 @@ void SideBar::update_data_inf()
     inf->SetDefaultStyle(boldattr);
     inf->AppendText(s2wx("@" + S(n) + "\n"));
     inf->SetDefaultStyle(defattr);
-    inf->AppendText(s2wx(AL->get_data(n)->getInfo()));
+    inf->AppendText(s2wx(ftk->get_data(n)->get_info()));
+    wxFileName fn(s2wx(ftk->get_data(n)->get_filename()));
+    if (fn.IsOk() && !fn.IsAbsolute())
+        inf->AppendText(wxT("\nPath: ") + fn.MakeAbsolute());
     inf->ShowPosition(0);
 }
 
@@ -705,7 +725,7 @@ void SideBar::update_func_inf()
     font.SetWeight(wxFONTWEIGHT_BOLD);
     boldattr.SetFont(font);
 
-    Function const* func = AL->get_function(active_function);
+    Function const* func = ftk->get_function(active_function);
     inf->SetDefaultStyle(boldattr);
     inf->AppendText(s2wx(func->xname));
     inf->SetDefaultStyle(defattr);
@@ -719,11 +739,13 @@ void SideBar::update_func_inf()
         inf->AppendText(wxT("\nFWHM: ") + s2wx(S(func->fwhm())));
     if (func->has_iwidth()) 
         inf->AppendText(wxT("\nInt. Width: ") + s2wx(S(func->iwidth())));
+    if (func->has_other_props()) 
+        inf->AppendText(wxT("\n") + s2wx(func->other_props_str()));
     vector<string> in;
-    for (int i = 0; i < AL->get_ds_count(); ++i) {
-        if (contains_element(AL->get_sum(i)->get_ff_idx(), active_function))
+    for (int i = 0; i < ftk->get_ds_count(); ++i) {
+        if (contains_element(ftk->get_sum(i)->get_ff_idx(), active_function))
             in.push_back("@" + S(i) + ".F");
-        if (contains_element(AL->get_sum(i)->get_zz_idx(), active_function))
+        if (contains_element(ftk->get_sum(i)->get_zz_idx(), active_function))
             in.push_back("@" + S(i) + ".Z");
     }
     if (!in.empty())
@@ -744,12 +766,12 @@ void SideBar::update_var_inf()
     font.SetWeight(wxFONTWEIGHT_BOLD);
     boldattr.SetFont(font);
 
-    Variable const* var = AL->get_variable(n);
+    Variable const* var = ftk->get_variable(n);
     inf->SetDefaultStyle(boldattr);
-    string t = var->xname + " = " + var->get_formula(AL->get_parameters());
+    string t = var->xname + " = " + var->get_formula(ftk->get_parameters());
     inf->AppendText(s2wx(t));
     inf->SetDefaultStyle(defattr);
-    vector<string> in = AL->get_variable_references(var->name);
+    vector<string> in = ftk->get_variable_references(var->name);
     if (!in.empty())
         inf->AppendText(s2wx("\nIn:\n    " + join_vector(in, "\n    ")));
     inf->ShowPosition(0);
@@ -763,14 +785,21 @@ void SideBar::add_variable_to_bottom_panel(Variable const* var,
     bp_statict.push_back(name_st);
     if (var->is_simple() || var->is_constant()) {
         FancyRealCtrl *frc = new FancyRealCtrl(bottom_panel, -1,
-                                               var->get_value(), var->xname,
-                                               !var->is_simple(), this);
-        frc->connect_to_onkeydown(wxKeyEventHandler(FFrame::focus_input), frame);
+                                               var->get_value(), 
+                                               s2wx(var->xname),
+                                               !var->is_simple(), 
+                            make_callback<FancyRealCtrl const*>().V1(this, 
+                                            &SideBar::on_changing_frc_value),
+                            make_callback<FancyRealCtrl const*>().V1(this, 
+                                            &SideBar::on_changed_frc_value),
+                            make_callback<FancyRealCtrl const*>().V1(this, 
+                                            &SideBar::on_toggled_frc_lock));
+        frc->ConnectToOnKeyDown(wxKeyEventHandler(FFrame::focus_input), frame);
         bp_sizer->Add(frc, 1, wxALL|wxEXPAND, 1);
         bp_frc.push_back(frc);
     }
     else {
-        string t = var->xname + " = " + var->get_formula(AL->get_parameters());
+        string t = var->xname + " = " + var->get_formula(ftk->get_parameters());
         wxStaticText *var_st = new wxStaticText(bottom_panel, -1, s2wx(t));
         bp_sizer->Add(var_st, 1, wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL, 1);
         bp_statict.push_back(var_st);
@@ -778,6 +807,29 @@ void SideBar::add_variable_to_bottom_panel(Variable const* var,
     }
 }
 
+/// draw function draft with XOR
+void SideBar::on_changing_frc_value(FancyRealCtrl const* frc)
+{
+    vector<fp> p_values(bp_func->nv);
+    for (int i = 0; i < bp_func->nv; ++i) {
+        string xname = "$" + bp_func->get_var_name(i);
+        p_values[i] = wx2s(frc->GetTip()) != xname ? bp_func->get_var_value(i) 
+                                                   : frc->GetValue();
+    }
+    frame->get_main_plot()->draw_xor_peak(bp_func, p_values);
+}
+
+void SideBar::on_changed_frc_value(FancyRealCtrl const* frc)
+{
+    ftk->exec(wx2s(frc->GetTip()) + " = ~" + wx2s(frc->GetValueStr()));
+}
+
+void SideBar::on_toggled_frc_lock(FancyRealCtrl const* frc)
+{
+    string vname = wx2s(frc->GetTip());
+    ftk->exec(vname + " = " + (frc->IsLocked() ? "{" : "~{") + vname + "}");
+}
+
 void SideBar::clear_bottom_panel()
 {
     for (vector<wxStaticText*>::iterator i = bp_statict.begin(); 
@@ -797,27 +849,16 @@ vector<bool> SideBar::make_bottom_panel_sig(Function const* func)
 {
     vector<bool> sig;
     for (int i = 0; i < size(func->type_var_names); ++i) {
-        Variable const* var = AL->get_variable(func->get_var_idx(i));
+        Variable const* var = ftk->get_variable(func->get_var_idx(i));
         sig.push_back(var->is_simple() || var->is_constant());
     }
     return sig;
 }
 
-void SideBar::draw_function_draft(FancyRealCtrl const* frc) const
-{
-    vector<fp> p_values(bp_func->nv);
-    for (int i = 0; i < bp_func->nv; ++i) {
-        string xname = "$" + bp_func->get_var_name(i);
-        p_values[i] = frc->get_name() != xname ? bp_func->get_var_value(i) 
-                                               : frc->get_value();
-    }
-    frame->get_main_plot()->draw_xor_peak(bp_func, p_values);
-}
-
 void SideBar::change_bp_parameter_value(int idx, double value)
 {
     if (idx < (int)bp_frc.size()) 
-        bp_frc[idx]->set_temporary_value(value); 
+        bp_frc[idx]->SetTemporaryValue(value); 
 }
 
 void SideBar::update_bottom_panel()
@@ -828,7 +869,7 @@ void SideBar::update_bottom_panel()
         return;
     }
     bottom_panel->Freeze();
-    bp_func = AL->get_function(active_function);
+    bp_func = ftk->get_function(active_function);
     wxString new_label = s2wx(bp_func->xname + " : " + bp_func->type_name);
     if (bp_label->GetLabel() != new_label)
         bp_label->SetLabel(new_label);
@@ -837,7 +878,7 @@ void SideBar::update_bottom_panel()
         clear_bottom_panel();
         bp_sig = sig;
         for (int i = 0; i < bp_func->nv; ++i) {
-            Variable const* var = AL->get_variable(bp_func->get_var_idx(i));
+            Variable const* var = ftk->get_variable(bp_func->get_var_idx(i));
             add_variable_to_bottom_panel(var, bp_func->type_var_names[i]);
         }
         int sash_pos = GetClientSize().GetHeight() - 3
@@ -851,16 +892,17 @@ void SideBar::update_bottom_panel()
             string const& t = bp_func->type_var_names[i];
             (*st)->SetLabel(s2wx(t));
             ++st;
-            Variable const* var = AL->get_variable(bp_func->get_var_idx(i));
+            Variable const* var = ftk->get_variable(bp_func->get_var_idx(i));
             if (var->is_simple() || var->is_constant()) {
-                bp_frc[i]->set(var->get_value(), var->xname);
-                if (bp_frc[i]->get_locked() != !var->is_simple())
-                    bp_frc[i]->toggle_lock(false);
+                bp_frc[i]->SetValue(var->get_value());
+                bp_frc[i]->SetTip(s2wx(var->xname));
+                if (bp_frc[i]->IsLocked() != !var->is_simple())
+                    bp_frc[i]->ToggleLock();
             }
             else {
                 assert (bp_frc[i] == 0);
                 string f = var->xname + " = " 
-                                   + var->get_formula(AL->get_parameters());
+                                   + var->get_formula(ftk->get_parameters());
                 (*st)->SetLabel(s2wx(f));
                 ++st;
             }
@@ -872,13 +914,13 @@ void SideBar::update_bottom_panel()
 
 bool SideBar::howto_plot_dataset(int n, bool& shadowed, int& offset) const
 {
+    if (ftk->get_ds_count() != d->list->GetItemCount())
+        d->update_data_list(true, true);
     // choice_idx: 0: "show all datasets" 
     //             1: "show only selected" 
     //             2: "shadow unselected" 
     //             3: "hide all"
     int choice_idx = data_look->GetSelection();
-    if (n >= d->list->GetItemCount())
-        d->update_data_list(true, true);
     bool sel = d->list->IsSelected(n) || d->list->GetFocusedItem() == n;
     if ((choice_idx == 1 && !sel) || choice_idx == 3)
         return false;
@@ -892,10 +934,10 @@ vector<string> SideBar::get_selected_func() const
     vector<string> dd;
     for (int i = f->list->GetFirstSelected(); i != -1; 
                                             i = f->list->GetNextSelected(i))
-        dd.push_back(AL->get_function(i)->xname);
+        dd.push_back(ftk->get_function(i)->xname);
     if (dd.empty() && f->list->GetItemCount() > 0) {
         int n = f->list->GetFocusedItem();
-        dd.push_back(AL->get_function(n == -1 ? 0 : n)->xname);
+        dd.push_back(ftk->get_function(n == -1 ? 0 : n)->xname);
     }
     return dd;
 }
@@ -905,28 +947,28 @@ vector<string> SideBar::get_selected_vars() const
     vector<string> dd;
     for (int i = v->list->GetFirstSelected(); i != -1; 
                                              i = v->list->GetNextSelected(i))
-        dd.push_back(AL->get_variable(i)->xname);
+        dd.push_back(ftk->get_variable(i)->xname);
     if (dd.empty() && v->list->GetItemCount() > 0) {
         int n = v->list->GetFocusedItem();
-        dd.push_back(AL->get_function(n == -1 ? 0 : n)->xname);
+        dd.push_back(ftk->get_function(n == -1 ? 0 : n)->xname);
     }
     return dd;
 }
 
 
-void SideBar::OnFuncFocusChanged(wxListEvent& WXUNUSED(event))
+void SideBar::OnFuncFocusChanged(wxListEvent&)
 {
     int n = f->list->GetFocusedItem(); 
     if (n == -1)
         active_function = -1;
     else {
         string name = wx2s(f->list->GetItemText(n));
-        active_function = AL->find_function_nr(name);
+        active_function = ftk->find_function_nr(name);
     }
     do_activate_function();
 }
 
-void SideBar::OnVarFocusChanged(wxListEvent& WXUNUSED(event))
+void SideBar::OnVarFocusChanged(wxListEvent&)
 {
     update_var_inf();
 }
